@@ -80,6 +80,7 @@ class BeamTrace():
         self.number_of_lenses = 0
         self.number_of_refractions = 0
 
+    ##### set functions #####
     def set_isCavity(self, isCavity):
         '''Sets the isCavity parameter to True or False.  Default is True'''
         self.isCavity = isCavity
@@ -90,6 +91,7 @@ class BeamTrace():
         self.q_input = q
         return
 
+    ##### get functions #####
     def get_q(self):
         '''Calculate q-parameter from ABCD matrix
         From Koji Arai T1300189:
@@ -116,7 +118,7 @@ class BeamTrace():
             multiplier = 180/np.pi
         else:
             multiplier = 1.0
-#         total_gouy = np.sign(B) * np.arccos(0.5 * (A + D))
+        # total_gouy = np.sign(B) * np.arccos(0.5 * (A + D))
         total_gouy = 2 * np.arccos(np.sign(B) * np.sqrt((A+D+2)/4))
         return multiplier * total_gouy
 
@@ -144,6 +146,8 @@ class BeamTrace():
         gouy = np.arctan2( np.real(q), np.imag(q) )
         return gouy
 
+
+    ##### Add component functions #####
     def add_space(self, length, index_of_refraction=1, name=None):
         '''Adds a space to the beam path
         '''
@@ -443,6 +447,41 @@ class BeamTrace():
         q_out = (A * q_in + B)/(C * q_in + D)
         return q_out
 
+    def calc_beam_waists(self):
+        '''Calculates the minimum beam waist for each space in the cavity.
+        Must be done after scan_cavity() has been run.
+        '''
+        self.beam_waist_locations = np.array([])
+        self.beam_waists = np.array([])
+
+        for key in self.comp_dict.keys():
+            if self.comp_dict[key]['type'] == 'space':
+                z_start = self.comp_dict[key]['start_location']
+                z_end = self.comp_dict[key]['end_location']
+
+                z_start_index = np.argwhere(self.zz > z_start)[0,0]
+                z_end_index = np.argwhere(self.zz <= z_end)[-1,0]
+
+                try:
+                    temp_beam_waist_index = np.argmin(self.ww[z_start_index:z_end_index])
+                except ValueError:
+                    # z_start_index == z_end_index, meaning the steps are too large for this space
+                    continue
+                beam_waist_index = z_start_index + temp_beam_waist_index
+
+                temp_beam_waist_location = self.zz[beam_waist_index]
+                temp_beam_waist = self.ww[beam_waist_index]
+
+                # check if "waist" is at the same location as the start of end of space, then it is no waist
+                if beam_waist_index == z_start_index or beam_waist_index == z_end_index - 1:
+                    continue
+
+                self.beam_waist_locations = np.append(self.beam_waist_locations, temp_beam_waist_location)
+                self.beam_waists = np.append(self.beam_waists, temp_beam_waist)
+        return
+
+
+
     def scan_cavity(self, steps=500, round_trip=False):
         '''Scans the cavity along the axis of propagation (aka z-axis) from input to end mirror.
         Inputs:
@@ -581,12 +620,31 @@ class BeamTrace():
             qq = np.append(qq, q_out)
             gouy = np.append(gouy, gouy_accumulated)
 
+        # store as attributes
+        self.zz = zz
+        self.ww = ww
+        self.gouy = gouy
+        self.qq = qq
+
+        # calculate the beam waists
+        self.calc_beam_waists()
+
         return zz, ww, gouy, qq
 
-    def plot_cavity_scan(self, steps=500, round_trip=False, color='C3', ls='-', label=None, fig=None):
+    def plot_cavity_scan(self, steps=500, round_trip=False, plot_beam_waists=True, color='C3', ls='-', label=None, fig=None):
+        '''Plot the cavity scan beam profile for beam waist and accumulated gouy phase.
+        '''
+        self.plot_beam_waists = plot_beam_waists
+
         zz, ww, gouy, qq = self.scan_cavity(steps=steps, round_trip=round_trip)
         names, locations = self.get_component_locations()
 
+        # switch to class-based attributes
+        zz = self.zz
+        ww = self.ww
+        gouy = self.gouy
+        qq = self.qq
+
         newFig = True
         if fig==None:
             fig, (s1, s2) = plt.subplots(2, figsize=(20,9))
@@ -602,6 +660,47 @@ class BeamTrace():
 
         s2.plot(zz, gouy*180/np.pi, ls=ls, color=color, label=label)
 
+        if self.plot_beam_waists:
+            beam_waists = 0
+            max_plot_ww = np.max(ww*1e3)
+            arrow_height = max_plot_ww * 0.75
+            for z_waist, w_waist in zip(self.beam_waist_locations, self.beam_waists):
+                beam_waists += 1
+                plot_w_waist = w_waist*1e3
+
+                # check if the arrow height is high enough
+                if arrow_height < plot_w_waist:
+                    text_height = 0
+                    plot_arrow_height = max_plot_ww
+                    verticalalignment = 'center'
+                else:
+                    text_height = arrow_height
+                    plot_arrow_height = arrow_height
+                    verticalalignment = 'bottom'
+
+                s1.annotate(f"Beam waist {beam_waists}\nLoc = {z_waist:.2f} m\nWaist = {plot_w_waist:.2f} mm",
+                    xy=(z_waist, plot_w_waist), xycoords='data',
+                    xytext=(z_waist, text_height), textcoords='data',
+                    fontsize=12,
+                    horizontalalignment='center',
+                    verticalalignment=verticalalignment
+                )
+                s1.annotate("",
+                    xy=(z_waist, plot_w_waist), xycoords='data',
+                    xytext=(z_waist, plot_arrow_height), textcoords='data',
+                    fontsize=10,
+                    arrowprops=dict(arrowstyle="->",
+                        connectionstyle="arc3"),
+                )
+                s1.annotate("",
+                    xy=(z_waist, -plot_w_waist), xycoords='data',
+                    xytext=(z_waist, -plot_arrow_height), textcoords='data',
+                    fontsize=10,
+                    arrowprops=dict(arrowstyle="->",
+                        connectionstyle="arc3"),
+                )
+            
+
         if newFig:
             for ii, name, location in zip(np.arange(len(names)), names, locations):
                 s1.axvline(x=location, ls='--', color='C{}'.format(ii), label=name)
